#include <autoconf.h>
#include <hermit.h>
#include <htypes.h>
#include <arch/system.h>
#include <arch/pinmux.h>
#include "pinmux.h"

#define read8(addr) \
	({ unsigned char __v = (*(volatile unsigned char *) (addr)); __v; })
#define read16(addr) \
	({ unsigned short __v = (*(volatile unsigned short *) (addr)); __v; })
#define read32(addr) \
	({ unsigned int __v = (*(volatile unsigned int *) (addr)); __v; })

#define write8(addr, b) \
	(void)((*(volatile unsigned char *) (addr)) = (b))
#define write16(addr, b) \
	(void)((*(volatile unsigned short *) (addr)) = (b))
#define write32(addr, b) \
	(void)((*(volatile unsigned int *) (addr)) = (b))

/* LBSC */
#define LBSC_CS0CTRL	0xfec00200
#define LBSC_CS1CTRL	0xfec00204
#define LBSC_ECS0CTRL	0xfec00208
#define LBSC_ECS1CTRL	0xfec0020c
#define LBSC_ECS2CTRL	0xfec00210
#define LBSC_ECS3CTRL	0xfec00214
#define LBSC_ECS4CTRL	0xfec00218
#define LBSC_ECS5CTRL	0xfec0021c
#define LBSC_CSWCR0	0xfec00230
#define LBSC_CSWCR1	0xfec00234
#define LBSC_ECSWCR0	0xfec00238
#define LBSC_ECSWCR1	0xfec0023c
#define LBSC_ECSWCR2	0xfec00240
#define LBSC_ECSWCR3	0xfec00244
#define LBSC_ECSWCR4	0xfec00248
#define LBSC_ECSWCR5	0xfec0024c
#define LBSC_EXDMAWCR0	0xfec00250
#define LBSC_EXDMAWCR1	0xfec00254
#define LBSC_EXDMAWCR2	0xfec00258
#define LBSC_CSPWCR0	0xfec00280
#define LBSC_CSPWCR1	0xfec00284
#define LBSC_ECSPWCR0	0xfec00288
#define LBSC_ECSPWCR1	0xfec0028c
#define LBSC_ECSPWCR2	0xfec00290
#define LBSC_ECSPWCR3	0xfec00294
#define LBSC_ECSPWCR4	0xfec00298
#define LBSC_ECSPWCR5	0xfec0029c
#define LBSC_EXWTSYNC	0xfec002a0
#define LBSC_CS0BSTCTL	0xfec002b0
#define LBSC_CS0BTPH	0xfec002b4
#define LBSC_CS1GDST	0xfec002c0
#define LBSC_ECS0GDST	0xfec002c4
#define LBSC_ECS1GDST	0xfec002c8
#define LBSC_ECS2GDST	0xfec002cc
#define LBSC_ECS3GDST	0xfec002d0
#define LBSC_ECS4GDST	0xfec002d4
#define LBSC_ECS5GDST	0xfec002d8
#define LBSC_EXDMASET0	0xfec002f0
#define LBSC_EXDMASET1	0xfec002f4
#define LBSC_EXDMASET2	0xfec002f8
#define LBSC_EXDMCR0	0xfec00310
#define LBSC_EXDMCR1	0xfec00314
#define LBSC_EXDMCR2	0xfec00318
#define LBSC_BCINTSR	0xfec00330
#define LBSC_BCINTCR	0xfec00334
#define LBSC_BCINTMR	0xfec00338
#define LBSC_EXBATLV	0xfec00340
#define LBSC_EXWTSTS	0xfec00344
#define LBSC_ATACSCTRL	0xfec00380
#define LBSC_EXBCT	0xfec003c0
#define LBSC_EXTCT	0xfec003c4
#define LBSC_EXTSR	0xfec00010
#define LBSC_EXTADR	0xfec00014


/* DBSC */
#define DBSC3_0_BASE		(0xe6790000)
#define DBSC3_1_BASE		(0xe67a0000)

#define DBSC3_DBSTATE1		(0x00c)
#define DBSC3_DBACEN		(0x010)
#define DBSC3_DBRFEN		(0x014)
#define DBSC3_DBCMD		(0x018)
#define DBSC3_DBWAIT		(0x01c)
#define DBSC3_DBKIND		(0x020)
#define DBSC3_DBCONF0		(0x024)
#define DBSC3_DBPHYTYPE		(0x030)
#define DBSC3_DBTR0		(0x040)
#define DBSC3_DBTR1		(0x044)
#define DBSC3_DBTR2		(0x048)
#define DBSC3_DBTR3		(0x050)
#define DBSC3_DBTR4		(0x054)
#define DBSC3_DBTR5		(0x058)
#define DBSC3_DBTR6		(0x05c)
#define DBSC3_DBTR7		(0x060)
#define DBSC3_DBTR8		(0x064)
#define DBSC3_DBTR9		(0x068)
#define DBSC3_DBTR10		(0x06c)
#define DBSC3_DBTR11		(0x070)
#define DBSC3_DBTR12		(0x074)
#define DBSC3_DBTR13		(0x078)
#define DBSC3_DBTR14		(0x07c)
#define DBSC3_DBTR15		(0x080)
#define DBSC3_DBTR16		(0x084)
#define DBSC3_DBTR17		(0x088)
#define DBSC3_DBTR18		(0x08c)
#define DBSC3_DBTR19		(0x090)
#define DBSC3_DBBL		(0x0b0)
#define DBSC3_DBADJ0		(0x0c0)
#define DBSC3_DBADJ2		(0x0c8)
#define DBSC3_DBRFCNF0		(0x0e0)
#define DBSC3_DBRFCNF1		(0x0e4)
#define DBSC3_DBRFCNF2		(0x0e8)
#define DBSC3_DBCALCNF		(0x0f4)
#define DBSC3_DBCALTR		(0x0f8)
#define DBSC3_DBRNK0		(0x100)
#define DBSC3_DBPDNCNF		(0x180)
#define DBSC3_DBDFISTAT		(0x240)
#define DBSC3_DBDFICNT		(0x244)
#define DBSC3_DBPDLCK		(0x280)
#define DBSC3_DBPDRGA		(0x290)
#define DBSC3_DBPDRGD		(0x2a0)
#define DBSC3_DBBS0CNT1		(0x304)
#define DBSC3_DBWT0CNF0		(0x380)
#define DBSC3_DBWT0CNF1		(0x384)
#define DBSC3_DBWT0CNF2		(0x388)
#define DBSC3_DBWT0CNF3		(0x38c)
#define DBSC3_DBWT0CNF4		(0x390)
#define DBSC3_DBSCHECNT0	(0x500)


/* board_type */
#define BOARD_ARMADILLOEVA1500	0

static void __init arch_dram_init(u32 base)
{
	/* 1-a-1 */
	write32(DBSC3_DBCMD + base, 0x21000000);
	write32(DBSC3_DBCMD + base, 0x11000000);
	write32(DBSC3_DBCMD + base, 0x10000000);

	/* 1-a-2 */
	write32(DBSC3_DBPDLCK + base, 0x0000a55a);

	/* Additional PHY Setting */
	write32(DBSC3_DBPDRGA + base, 0x00000010);
	write32(DBSC3_DBPDRGD + base, 0xf004649b);

	/* ODT on */
	write32(DBSC3_DBPDRGA + base, 0x0000000f);
	write32(DBSC3_DBPDRGD + base, 0x00181885);
	write32(DBSC3_DBPDRGA + base, 0x00000070);
	write32(DBSC3_DBPDRGD + base, 0x7c000887);
	write32(DBSC3_DBPDRGA + base, 0x00000080);
	write32(DBSC3_DBPDRGD + base, 0x7c000887);
	write32(DBSC3_DBPDRGA + base, 0x00000090);
	write32(DBSC3_DBPDRGD + base, 0x7c000887);
	write32(DBSC3_DBPDRGA + base, 0x000000a0);
	write32(DBSC3_DBPDRGD + base, 0x7c000887);
	write32(DBSC3_DBPDRGA + base, 0x000000b0);
	write32(DBSC3_DBPDRGD + base, 0x7c000880);
	write32(DBSC3_DBPDRGA + base, 0x000000c0);
	write32(DBSC3_DBPDRGD + base, 0x7c000880);
	write32(DBSC3_DBPDRGA + base, 0x000000d0);
	write32(DBSC3_DBPDRGD + base, 0x7c000880);
	write32(DBSC3_DBPDRGA + base, 0x000000e0);
	write32(DBSC3_DBPDRGD + base, 0x7c000880);

	/* 1-b */
	write32(DBSC3_DBKIND + base, 0x00000007);
	write32(DBSC3_DBCONF0 + base, 0x0f030a02);
	write32(DBSC3_DBPHYTYPE + base, 0x00000001);
	write32(DBSC3_DBBL + base, 0x00000000);
	write32(DBSC3_DBTR0 + base, 0x0000000b);
	write32(DBSC3_DBTR1 + base, 0x00000008);
	write32(DBSC3_DBTR2 + base, 0x00000000);
	write32(DBSC3_DBTR3 + base, 0x0000000b);
	write32(DBSC3_DBTR4 + base, 0x000c000b);
	write32(DBSC3_DBTR5 + base, 0x00000027);
	write32(DBSC3_DBTR6 + base, 0x0000001c);
	write32(DBSC3_DBTR7 + base, 0x00000006);
	write32(DBSC3_DBTR8 + base, 0x00000020);
	write32(DBSC3_DBTR9 + base, 0x00000008);
	write32(DBSC3_DBTR10 + base, 0x0000000c);
	write32(DBSC3_DBTR11 + base, 0x00000009);
	write32(DBSC3_DBTR12 + base, 0x00000012);
	write32(DBSC3_DBTR13 + base, 0x000000d0);
	write32(DBSC3_DBTR14 + base, 0x00140005);
	write32(DBSC3_DBTR15 + base, 0x00050004);
	write32(DBSC3_DBTR16 + base, 0x70233005);
	write32(DBSC3_DBTR17 + base, 0x000c0000);
	write32(DBSC3_DBTR18 + base, 0x00000300);
	write32(DBSC3_DBTR19 + base, 0x00000040);
	write32(DBSC3_DBRNK0 + base, 0x00000001);
	write32(DBSC3_DBADJ0 + base, 0x00020001);
	write32(DBSC3_DBADJ2 + base, 0x20042004);
	write32(DBSC3_DBWT0CNF0 + base, 0x00020002);
	write32(DBSC3_DBWT0CNF4 + base, 0x0000001f);

	/* 1-c-1 */
	while (!(read32(DBSC3_DBDFISTAT + base) & 0x00000001));
	/* 1-c-2 */
	write32(DBSC3_DBDFICNT + base, 0x00000011);
	/* 1-c-3 */
	write32(DBSC3_DBPDRGA + base, 0x00000006);
	/* 1-c-4 */
	write32(DBSC3_DBPDRGD + base, 0x0001c000);
	/* 1-c-5 */
	write32(DBSC3_DBPDRGA + base, 0x00000003);
	/* 1-c-6 */
	write32(DBSC3_DBPDRGD + base, 0x0300c481);
	/* 1-c-7 */
	write32(DBSC3_DBPDRGA + base, 0x00000023);
	/* 1-c-8 */
	write32(DBSC3_DBPDRGD + base, 0x00fdb6c0);
	/* 1-c-9 */
	write32(DBSC3_DBPDRGA + base, 0x00000011);
	/* 1-c-10 */
	write32(DBSC3_DBPDRGD + base, 0x1000040b);
	/* 1-c-11 */
	write32(DBSC3_DBPDRGA + base, 0x00000012);
	/* 1-c-12 */
	write32(DBSC3_DBPDRGD + base, 0x9d9cbb66);
	/* 1-c-13 */
	write32(DBSC3_DBPDRGA + base, 0x00000013);
	/* 1-c-14 */
	write32(DBSC3_DBPDRGD + base, 0x1a868400);
	/* 1-c-15 */
	write32(DBSC3_DBPDRGA + base, 0x00000014);
	/* 1-c-16 */
	write32(DBSC3_DBPDRGD + base, 0x300214d8);
	/* 1-c-17 */
	write32(DBSC3_DBPDRGA + base, 0x00000015);
	/* 1-c-18 */
	write32(DBSC3_DBPDRGD + base, 0x00000d70);
	/* 1-c-19 */
	write32(DBSC3_DBPDRGA + base, 0x00000016);
	/* 1-c-20 */
	write32(DBSC3_DBPDRGD + base, 0x00000006);
	/* 1-c-21 */
	write32(DBSC3_DBPDRGA + base, 0x00000017);
	/* 1-c-22 */
	write32(DBSC3_DBPDRGD + base, 0x00000018);
	/* 1-c-23 */
	write32(DBSC3_DBPDRGA + base, 0x0000001a);
	/* 1-c-24 */
	write32(DBSC3_DBPDRGD + base, 0x910035c7);
	/* 1-c-25 */
	write32(DBSC3_DBPDRGA + base, 0x00000004);
	/* 1-c-26 */
	while (!(read32(DBSC3_DBPDRGD + base) & 0x00000001));
	/* 1-c-27 */
	write32(DBSC3_DBPDRGA + base, 0x00000001);
	/* 1-c-28 */
	write32(DBSC3_DBPDRGD + base, 0x00000181);
	/* 1-c-29 */
	write32(DBSC3_DBCMD + base, 0x11000000);
	/* 1-c-30 */
	write32(DBSC3_DBPDRGA + base, 0x00000004);
	/* 1-c-31 */
	while (!(read32(DBSC3_DBPDRGD + base) & 0x00000001));
	/* 1-c-32 */
	write32(DBSC3_DBPDRGA + base, 0x00000001);
	/* 1-c-33 */
	write32(DBSC3_DBPDRGD + base, 0x0000fe01);
	/* 1-c-34 */
	write32(DBSC3_DBPDRGA + base, 0x00000004);
	/* 1-c-35 */
	while (!(read32(DBSC3_DBPDRGD + base) & 0x00000001));

	/* 1-d */
	write32(DBSC3_DBBS0CNT1 + base, 0x00000000);
	write32(DBSC3_DBCALCNF + base, 0x01004c20);
	write32(DBSC3_DBCALTR + base, 0x014000aa);
	write32(DBSC3_DBRFCNF0 + base, 0x00000140);
	write32(DBSC3_DBRFCNF1 + base, 0x00081860);
	write32(DBSC3_DBRFCNF2 + base, 0x00010000);
	write32(DBSC3_DBRFEN + base, 0x00000001);
	write32(DBSC3_DBACEN + base, 0x00000001);
	write32(DBSC3_DBPDLCK + base, 0x00000000);
}

extern void __init qos_init(void);

void __init arch_init(void)
{
	/* QOS */
	qos_init();

	/* DBSC3 */
	arch_dram_init(DBSC3_0_BASE);
	arch_dram_init(DBSC3_1_BASE);
}
